from cleo import Application

from .commands import DeployCommand, LoginCommand

application = Application()
application.add(DeployCommand())
application.add(LoginCommand())


if __name__ == "main":
    application.run()



import os
import json

from pathlib import Path
from cleo import Command

from ...auth import Auth


class LoginCommand(Command):
    """
    Login

    login
        {token : Token to connect}
        {backend=https://elasticedge.dev/ : The service to connect to}
    """

    def handle(self):
        token = self.argument("token")
        backend = self.argument("backend")

        Auth.write(token, backend)

        self.line(f"Login authentication saved", "info")



import time
import os
import yaml

from pathlib import Path
from cleo import Command

from ...api import deploy, healthcheck
from ...api.exceptions import GraphQLError


class DeployCommand(Command):
    """
    Deploy

    deploy
        {spec=elasticedge.yaml : The spec file with information about what to deploy}
    """

    def handle(self):
        spec = self.argument("spec")
        spec_file = os.path.join(Path.cwd(), spec)

        if os.path.isdir(spec_file):
            spec_file = os.path.join(spec_file, "elasticedge.yaml")

        if not os.path.exists(spec_file):
            self.line_error(f"Spec file {spec_file} not found!", "error")
            return

        with open(spec_file) as f:
            configuration = yaml.load(f, Loader=yaml.FullLoader)

        configuration["project"]["env"] = self.resolve_environment(
            configuration["project"]["env"]
        )

        for service in configuration["services"]:
            service_env = service["env"]
            service["env"] = self.resolve_environment(service_env)

        project_id = configuration["project"]["id"]
        name = configuration["project"]["name"]

        self.line(f"Deploying configuration to {name} ({project_id})")

        try:
            deploy(configuration)
        except GraphQLError as e:
            if e.response.status_code >= 500:
                self.line_error(
                    "Deployer answer was a 5xx. Checking if the deployment completed with an healthcheck after 3 seconds."
                )

                time.sleep(3)

                if healthcheck(configuration):
                    self.line_error(
                        f"The service is still up, but we had a few errors:", "error"
                    )
                else:
                    self.line_error("Healthcheck failed.")
            else:
                self.line_error(f"Unable to deploy new configuration", "error")

            for error in e.errors:
                self.line_error(f"[GraphQL Error] {error}", "error")

            exit(1)

        self.line(f"New configuration uploaded and deploy requested!", "info")

    def resolve_environment(self, envs):
        if not envs:
            return []

        new_env = []

        for variable in envs:
            new_env.append({"name": variable, "value": os.getenv(variable, "")})

        return new_env



import requests

from ..auth import Auth

from .exceptions import GraphQLError


def request(query, variables={}):
    auth = Auth.load()

    response = requests.post(
        f"{auth.backend}graphql/",
        json={"query": query.strip(), "variables": variables},
        headers={"Authorization": auth.token},
    )

    if response.status_code != 200:
        raise GraphQLError(response)

    return response



from .graphql import request


def deploy(configuration):
    return request(
        """
    mutation($configuration: ConfigurationInput!) {
        deploy(configuration: $configuration)
    }
    """,
        {"configuration": configuration},
    )



import time

from .graphql import request

import requests


def healthcheck(configuration):
    response = requests.get(f'https://{configuration["project"]["domain"]}')
    return response.status_code == 200



import dataclasses
import json
import os

from pathlib import Path


@dataclasses.dataclass
class Auth:
    FILE_PATH: str = dataclasses.field(
        default=os.path.join(Path.cwd(), "auth.json"), init=False
    )

    token: str
    backend: str

    @classmethod
    def load(cls) -> "Auth":
        with open(cls.FILE_PATH, "r") as f:
            contents = json.load(f)

            token = contents["token"]
            backend = contents["backend"]

        auth = cls(token=token, backend=backend)
        return auth

    @classmethod
    def write(cls, token: str, backend: str) -> None:
        data = {"token": token, "backend": backend}

        with open(cls.FILE_PATH, "w") as f:
            f.write(json.dumps(data))



class GraphQLError(ValueError):
    def __init__(self, response):
        self.response = response

        try:
            body = response.json()
            self.errors = [error["message"] for error in body["errors"]]
        except:
            self.errors = ["Internal error"]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['elasticedge',
 'elasticedge.api',
 'elasticedge.auth',
 'elasticedge.console',
 'elasticedge.console.commands']

package_data = \
{'': ['*']}

install_requires = \
['cleo>=0.7.6,<0.8.0', 'pyyaml>=5.2,<6.0', 'requests>=2.22.0,<3.0.0']

entry_points = \
{'console_scripts': ['elasticedge = elasticedge.console:main']}

setup_kwargs = {
    'name': 'elasticedge',
    'version': '0.5.0',
    'description': '',
    'long_description': None,
    'author': 'Marco Acierno',
    'author_email': 'marcoaciernoemail@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
